/* -*- C++ -*- Time-stamp: <11 Mar 01 11:10:16 Michael Bischoff> */

/*****************************************************************************/
/*									     */
/*									     */
/*	QMatchIt version 1.0 -- file game.cpp				     */
/*									     */
/*	Include file for the field base class of QMatchIt		     */
/*	written by Michael Bischoff (Michael.Bischoff@gmx.net)		     */
/*	November-2000							     */
/*	see COPYRIGHT.qmatchit for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/

#include "mainwindow.h"
#include <qtimer.h>
#include <qcanvas.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qstring.h>
#include <qevent.h>
#include <qmessagebox.h>

#include "canvas.h"
// #include "fields.h"
#include "game.h"

Mainwindow::Mainwindow(QWidget *parent, const char *name, WFlags f)
: QMainWindow(parent, name, f), lv_filename("tutorial.dat") {
    setCaption("QMatchIt");
    g = new game();
    level = 0;
    timer = new QTimer();
    connect(timer, SIGNAL(timeout()), this, SLOT(animate()));
    edit_mode = FALSE;

    view = new QCanvasView(QMatchItGraphics::getCanvas(), this);


    QMenuBar *menu = menuBar();

    file = new QPopupMenu;
    file->insertItem("Try &again", this, SLOT(sameLevel()), CTRL+Key_A);
    file->insertItem("&New level", this, SLOT(nextLevel()), CTRL+Key_N);
    file->insertSeparator();
    erase_id = file->insertItem("&Erase level", this, SLOT(clear()), CTRL+Key_E);
    file->insertItem("&Save level", this, SLOT(save()), CTRL+Key_S);
    file->insertItem("&Load level", this, SLOT(load()), CTRL+Key_L);
    file->insertItem("E&xit", qApp, SLOT(quit()), CTRL+Key_Q);
    menu->insertItem("&File", file);
    file->setItemEnabled(erase_id, FALSE);	// disabled since not edit mode

    options = new QPopupMenu;
    options->setCheckable(TRUE);
    dbf_id = options->insertItem("Double buffer", this, SLOT(toggleDoubleBuffer()));
    options->setItemChecked(dbf_id, TRUE);
    edt_id = options->insertItem("Edit Mode", this, SLOT(toggleEditMode()));
    options->setItemChecked(edt_id, FALSE);
    menu->insertItem("&Options",options);

    menu->insertSeparator();

    QPopupMenu *help = new QPopupMenu;
    help->insertItem("&About Qt", this, SLOT(aboutQt()), Key_F1);
    help->insertItem("&About QMatchIt", this, SLOT(aboutQMatchIt()), Key_F2);
    help->insertItem("&QMatchIt keys", this, SLOT(aboutKeys()), Key_F3);
    help->insertItem("&QMatchIt level editor", this, SLOT(aboutEditor()), Key_F4);
    menu->insertItem("&Help", help);

    statusBar();

    setCentralWidget(view);

    timer->start(100);
}

Mainwindow::~Mainwindow() {
    delete timer;
    delete view;
    delete g;
    // TODO: invoke destructor of superclass?
}

void Mainwindow::setLevelFile(const char *filename, int lv) {
    lv_filename = QString::fromUtf8(filename);
    level = lv;
}

void Mainwindow::setSpeed(int arg) {
    timer->changeInterval(arg);
}

void Mainwindow::animate() {
    if (edit_mode) {
	game::advance_all();
	return;
    }

    g->animate();
    if (!advanceLevel && g->levelIsSolved())
	advanceLevel = true;
}

void Mainwindow::save()  { g->save("level.dat"); }
void Mainwindow::load()  { g->load("level.dat"); }
void Mainwindow::clear() { g->clear(); }


void Mainwindow::nextLevel() {
    ++level;
    sameLevel();
}

void Mainwindow::sameLevel() {
    clear();
    advanceLevel = false;
    g->loadFromFile(lv_filename.utf8(), level);
    advanceLevel = false;

    QMessageBox *info = new QMessageBox(QString(g->getLvTitle()),
					QString(g->getLvDesc()),
					QMessageBox::Information,
					QMessageBox::Ok | QMessageBox::Default,
					0, 0, this);
    info->exec();		// show will return immediately in Qt 3.0
    delete info;
}

void Mainwindow::aboutQMatchIt() {
    static QMessageBox* about = new QMessageBox( "About the QMatch-It Hack",
	"This is the first try for the QMatch-It! game.",
	QMessageBox::Information, 1, 0, 0, this, 0, false);
    about->setButtonText(1, "Dismiss");
    about->show();
}

void Mainwindow::aboutQt() {
    QMessageBox::aboutQt( this, "Qt Match-It!" );
}

void Mainwindow::aboutKeys() {
    static QMessageBox* about = new QMessageBox( "QMatch-It Keys Help",
	"This is a key driven game. Forget about the mouse.<br>Please use the arrow keys to move the cursor. Engage or disengage the cursor with the boxes (the squares with the suit symbols).<p>Objective is to move identical boxes together to remove them.",
	QMessageBox::Information, 1, 0, 0, this, 0, false);
    about->setButtonText(1, "Dismiss");
    about->show();
}

void Mainwindow::aboutEditor() {
    static QMessageBox* about = new QMessageBox( "QMatch-It Editor Help",
	"There is a simple level editor that comes with the game.<br>\
You can paint a level by moving the cursor and pressing certain keys \
to create the corresponding tile at that place. The keys are as follows:\
<p><table>\
<tr><td>space</td><td>empty</td></tr>\
<tr><td>0</td><td>light weight (5)</td></tr>\
<tr><td>1..8</td><td>box</td></tr>\
<tr><td>B</td><td>bomb</td></tr>\
<tr><td>G</td><td>heavy weight (10)</td></tr>\
<tr><td>D,M,L,R</td><td>magnet (down, up, left, right)</td></tr>\
<tr><td>I</td><td>inverter for magnets</td></tr>\
<tr><td>S</td><td>safe ground</td></tr>\
<tr><td>T,Z,Y</td><td>teleporter</td></tr>\
<tr><td>U</td><td>unsafe ground</td></tr>\
<tr><td>V</td><td>vanishing ground</td></tr>\
<tr><td>W</td><td>wall / ground</td></tr>\
<tr><td>X</td><td>removable wall</td></tr>\
<tr><td>!,_,&amp;</td><td>lift</td></tr>\
<tr><td>&lt;,&gt;</td><td>oneway left, right</td></tr>\
<tr><td>^</td><td>balloon</td></tr>\
</table>\
Use <b>save level</b> from the file menu to save it a the file level.dat. You can add this file to a series file (like <b>tutorial.dat</b>) later.",
	QMessageBox::Information, 1, 0, 0, this, 0, false);
    about->setButtonText(1, "Dismiss");
    about->show();
}

void Mainwindow::toggleDoubleBuffer() {
    bool s = !options->isItemChecked(dbf_id);
    options->setItemChecked(dbf_id,s);
    QMatchItGraphics::getCanvas()->setDoubleBuffering(s);
}

void Mainwindow::toggleEditMode() {
    edit_mode = !edit_mode;
    options->setItemChecked(edt_id, edit_mode);
    file->setItemEnabled(erase_id, edit_mode);	// enabled if edit mode
}

void Mainwindow::keyPressEvent(QKeyEvent *qe) {
    //	cout << "main key pressed " << qe->key() << endl;
    if (!edit_mode) {
	if (advanceLevel) {
	    if (level) {	// previous level is not just startup
		int result;
		QMessageBox *win = new QMessageBox( "Level solved!",
						   QString("<h3>Level solved!</h3>Congratulations, you solved this level.<p>Your score for this level is ")
						   + QString::number(g->getScore())
						   + QString(".<p>Press OK to proceed to the next one."),
						   QMessageBox::Information,
						   QMessageBox::Ok | QMessageBox::Default,
						   QMessageBox::Retry, 0, this);
		win->exec();	// show() will return immediately in Qt 3.0
		result = win->result();
		delete win;
		if (result == QMessageBox::Ok)
		    nextLevel();
		else
		    sameLevel();
	    } else {
		// from level 0 you may only advance and have no score!
		nextLevel();
	    }
	    return;		// and skip key!
	}
	// key pressed and not in edit mode

	switch (qe->key()) {
	case Key_Space:
	    g->gameKey(game::Key_Select);
	    break;
	case Key_Left:
	    g->gameKey(game::Key_Left);
	    break;
	case Key_Right:
	    g->gameKey(game::Key_Right);
	    break;
	case Key_Up:
	    g->gameKey(game::Key_Up);
	    break;
	case Key_Down:
	    g->gameKey(game::Key_Down);
	    break;
	}
	return;
    }
    if (qe->key() >= 0x100) {
	switch (qe->key()) {
	case Key_Left:
	    g->editKey(game::Key_Left);
	    break;
	case Key_Right:
	    g->editKey(game::Key_Right);
	    break;
	case Key_Up:
	    g->editKey(game::Key_Up);
	    break;
	case Key_Down:
	    g->editKey(game::Key_Down);
	    break;
	}
    } else {
	g->editKey(qe->key());
    }
}
